package CompilerProj.Parser;

import CompilerProj.Lexical.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;

import java.util.*;


/** This class represents the added syntactic unit:
  **
  ** argument-list = expression { , expression }        */

public class ArgumentList extends SyntacticUnit {
    
    
    public static final TokenSet STARTS = ...;
    
    
    private Parameters paramList;  // parameter list for method
    
    
    public ArgumentList ( Scope c, Parameters p, TokenSet s ) {
        
        super(c,s);
        paramList = p;
        
    }; // constructor
    
    
    public void parse ( ) {
        
        Iterator<ADeclaration>  it;    // parameter list iterator
        Expression              anExpression; // an argument
        AVariable               formal;   // formal parameter declaration
        
        it = paramList.iterator();
        while ( true ) {
            anExpression = new Expression(...);
            anExpression.parse();
            if ( it.hasNext() ) {
                formal = (AVariable) it.next();
                if ( ! formal.getType().compatibleWith(anExpression.getType()) ) {
                    semanticError("Argument incompatible with parameter");
                };
            }
            else {
                semanticError("Too many arguments");
            };
            check(...);
        if ( tokenIn(...except(Expression.STARTS,COMMA)) ) break;
            expect(COMMA,"Unexpected symbol",oneOf(stopSet,Expression.STARTS));
        };
        if ( ! (paramList instanceof DummyParams ) && it.hasNext() ) {
            semanticError("Insufficient arguments");
        };
        
    }; // parse
    
    
} // ArgumentList