package CompilerProj;

import CompilerProj.IO.*;
import CompilerProj.Lexical.*;
import CompilerProj.Parser.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;

import javax.swing.*;
import java.io.*;
import java.util.*;

/** This class is the compiler. It opens the files, creates a ClassDcl object
  ** to parse the file and then closes files and terminates.     */

public class Compiler {
    
    
    public static final String  missingName = "*missing*"; // missing identifier
    
    public static TokenReader  source;   // source file
    public static ListingWriter  listing;  // listing file
    public static CodeWriter  code;   // code file
    public static Global   global;   // global declarations
    
    
    /** The class is never instantiated.        */
    
    private Compiler ( ) { };
    
    
    /** The main method opens the files and then parses a ClassDcl with
      ** stopSet {EOT}.
      **
      ** @param args file name to compile.       */
    
    public static void main( String args[] ) {
        
        File   sourceFile;  // source file
        File  directory;  // program home directory
        ClassDcl  aClass;   // class declaration being compiled
        
               :
            directory = sourceFile.getParentFile();
            global = Global.getGlobal(directory);
            aClass = new ClassDcl(global,oneOf(EOT));
            aClass.parse();
               :
            try {
                if ( listing.getNumErr() == 0 ) {
                    global.write(directory);
                };
            }
            catch ( IOException e ) {
                System.out.println("Unable to create symbol table file."+e);
            };
            code.writeLine(global.toString());
            source.close();
            listing.close();
            code.close();
                :
    }; // main
    
    
} // Compiler
