package CompilerProj.Parser;

import CompilerProj.Lexical.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;


/** This class represents the added syntactic unit:
  **
  ** if-stmt = if expression then { statement } [ else { statement } ] end ; */

public class IfStmt extends SyntacticUnit {
    
    
    public static final TokenSet STARTS = ...;
    
    
    public IfStmt ( Scope c, TokenSet s ) {
        
        super(c,s);
        
    }; // constructor
    
    
    public void parse ( ) {
        
        Expression anExpression;  // the condition
        Statement aStmt;    // a statement
        
        expect(IF,...);
        anExpression = new Expression(...));
        anExpression.parse();
        if ( ! (typeBool.compatibleWith(anExpression.getType())) ) {
            semanticError("Incompatible type");
        };
        expect(THEN,...);
        while ( true ) {
            check(...);
        if ( tokenIn(...except(Statement.STARTS)) ) break;
            if ( tokenIn(Statement.STARTS) ) {
                aStmt = new Statement(...);
                aStmt.parse();
            };
        };
        check(...);
        if ( tokenIs(ELSE) ) {
            accept();
            while ( true ) {
                check(...);
            if ( tokenIn(...except(Statement.STARTS)) ) break;
                if ( tokenIn(Statement.STARTS) ) {
                    aStmt = new Statement(...);
                    aStmt.parse();
                };
            };
        };
        expect(END,...);
        expect(SEMI,...);
        
    }; // parse
    
    
} // IfStmt