package CompilerProj.Parser;

import CompilerProj.Lexical.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;


/** This class represents the syntactic unit:
  **
  ** name      = qualifier [ . identifier ]         */

public class Name extends SyntacticUnit {
    
    
    public static final TokenSet STARTS = ...;
    
    
    private ADeclaration declaration; // declaration of the name
    private boolean   qualified;      // name is qualified
    private boolean   superQual;      // name contains super
    
    
    public Name ( Scope c, TokenSet s ) {
        
        super(c,s);
        declaration = universalDcl;
        qualified = false;
        superQual = false;
        
    }; // constructor
    
    
    public ADeclaration getDecl ( ) {...};  // declaration of name
    
    
    public boolean isQualified ( ) {...};  // name is qualified
    
    
    public boolean hasSuper ( ) {...};  // qualifier (or name) is super
    
    
    public void parse ( ) {
        
        Qualifier  aQualifier;  // qualifier
        AVariable  qualifier;  // qualifier as a variable
        String   name;
        Scope   newScope;
        
        aQualifier = new Qualifier(...);
        aQualifier.parse();
        check(...);
        if ( tokenIs(DOT) ) { // qualified
            qualified = true;
            if ( aQualifier.isSuper() ) {
                superQual = true;
                newScope = asScope(aQualifier.getDeclaration());
            }
            else {
                qualifier = asVarDcl(aQualifier.getDeclaration());
                newScope = asScope(qualifier.getType());
            };
            accept();
            name = expectId(...);
            declaration = findDcl(name,newScope);
        }
        else { // not qualified
            if ( aQualifier.isSuper() ) {
                superQual = true;
                declaration = asClassDcl(aQualifier.getDeclaration()).getConstructor();
            }
            else {
                declaration = aQualifier.getDeclaration();
            };
        };
        
    }; // parse
    
    
} // Name