package CompilerProj.Parser;

import CompilerProj.Lexical.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;


/** This class represents a generic syntactic unit. It provides standard methods
  ** in support of parsing, semantic analysis and code generation. Every syntactic
  ** unit has a scope and a set of stop tokens for parsing.      */

public abstract class SyntacticUnit {
    
    
    protected Scope   scope;
    protected TokenSet  stopSet;
    
    
    protected SyntacticUnit ( Scope c, TokenSet s ) {
        
        scope = c;
        stopSet = s;
        
    }; // constructor
    
    
    public abstract void parse ( ) ;
    
    
// Syntactic Analysis Helper Methods
    
    
// Semantic Analysis helper methods -- Scope checking
    
    protected String expectId ( String errMsg, TokenSet context ) {
        
        String name;
        
        if ( tokenIs(IDENTIFIER) ) {
            name = source.getCurToken().getSource();
        }
        else {
            name = missingName;
        };
        expect(IDENTIFIER,errMsg,context);
        return name;
        
    }; // expectId
    
    
    protected ADeclaration findDcl ( String id, Scope in ) {
        
        ADeclaration result;
        
        result = in.find(id);
        if ( result == null ) {
            semanticError("Identifier not declared");
            result = new Declarations(id);
            in.define(result);    // declare at this level
        };
        return result;
        
    }; // findDcl
    
    
    protected void checkAndDefine ( ADeclaration dcl ) {
        
        if ( ! dcl.getIdentifier().equals(missingName) ) {
            if ( ! scope.alreadyDefined(dcl.getIdentifier()) ) {
                scope.define(dcl);
            }
            else {
                semanticError("Duplicate declaration");
            };
        };
        
    }; // checkAndDefine
    
    
    protected void semanticError( String errMsg ) {
        
        listing.writeError(errMsg,source.getCurToken().getPosn());
        
    }; // semanticError
    
    
} // SyntacticUnit